/**
 * 
 */
package org.processmining.plugins.pnalignanalysis.visualization.patternmining;

import info.clearthought.layout.TableLayout;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.ListModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.event.ListSelectionListener;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.framework.util.ui.widgets.WidgetColors;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.petrinet.visualization.AlignmentConstants;

import com.fluxicon.slickerbox.components.RoundedPanel;
import com.fluxicon.slickerbox.factory.SlickerFactory;
import com.fluxicon.slickerbox.ui.SlickerScrollBarUI;

/**
 * @author aadrians Jul 16, 2012
 * 
 */
public class FreqItemsetsPanel extends JPanel {
	private static final long serialVersionUID = -4629608300018054960L;

	// reference to panel
	private final PNLogReplayPatternMinePanel parent;

	// selected frequent itemsets to be visualized
	private JComboBox selectionCbBox;

	// info about a selected itemset
	private ProMList2 itemsetsLst;
	private DefaultListModel listModel;
	private JLabel supportLbl;

	// selected movements to be visualized
	private JComboBox selectedMovementCbBox;
	private ProMList2 movementsLst;
	private DefaultListModel movementsModel;

	/**
	 * Encoded transitions and event classes
	 */
	private Transition[] int2trans = null;
	private XEventClass[] int2ec = null;

	private ItemSets itemSets = null;

	/**
	 * 
	 * @param int2trans
	 * @param int2ec
	 */
	public FreqItemsetsPanel(final PNLogReplayPatternMinePanel parent) {
		this.parent = parent;
		this.int2trans = parent.getInt2Trans();
		this.int2ec = parent.getInt2Ec();

		SlickerFactory factory = SlickerFactory.instance();

		// add all patterns to list selection
		// detail of selected list is shown using method updateStats()
		Object[] patterns = new Object[] { "No pattern mined" };
		selectionCbBox = factory.createComboBox(patterns);
		selectionCbBox.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				if ((itemSets != null) && (itemSets.getNumItemSets() > 0) && (selectionCbBox.getSelectedIndex() >= 0)) {
					updateItemSetSelect(selectionCbBox.getSelectedIndex());
				} else {
					updateItemSetSelect(-1);
				}
			}
		});

		// init list of frequent itemsets
		listModel = new DefaultListModel();
		itemsetsLst = new ProMList2("Frequent Itemsets", listModel);

		// init combobox of movements
		selectedMovementCbBox = factory.createComboBox(new Object[] {});
		for (int i = 0; i < int2trans.length; i++) {
			selectedMovementCbBox.addItem("[L/M] " + int2trans[i].getLabel());
		}
		for (int i = 0; i < int2trans.length; i++) {
			selectedMovementCbBox.addItem("[M] " + int2trans[i].getLabel());
		}
		for (int i = 0; i < int2ec.length; i++) {
			selectedMovementCbBox.addItem("[L] " + int2ec[i].getId());
		}

		AlignmentRenderer renderer = new AlignmentRenderer();
		itemsetsLst.getJList().setCellRenderer(renderer);

		selectedMovementCbBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				movementsModel.clear();

				// find related itemsets
				int selectedIndex = selectedMovementCbBox.getSelectedIndex();
				if ((selectedIndex >= 0) && (itemSets != null)) {
					// find all itemsets that contains the selected movement
					int numItemsets = itemSets.getNumItemSets();
					for (int i = 0; i < numItemsets; i++) {
						int[] items = itemSets.getItemSets(i);

						for (int j = 0; j < items.length; j++) {
							if (selectedIndex == items[j]) {
								movementsModel.addElement((i + 1) + ". (sup: " + itemSets.getSupport(i) + ")"
										+ getItemSetStr(i));
								break;
							}
						}
					}
				}
			}
		});

		// init list of involved itemsets
		movementsModel = new DefaultListModel();
		movementsLst = new ProMList2("Involved itemsets", movementsModel);

		movementsLst.getJList().setCellRenderer(renderer);

		// add GUI
		double[][] size = new double[][] { { 150, 350 }, { 30, 30, 200, 30, 200, 30 } };
		setLayout(new TableLayout(size));
		add(factory.createLabel("Select frequent itemsets"), "0,0");
		add(selectionCbBox, "1,0");
		add(factory.createLabel("Support"), "0,1");
		supportLbl = factory.createLabel("-");
		add(supportLbl, "1,1");

		add(itemsetsLst, "0,2,1,2");

		add(factory.createLabel("Select movements"), "0,3");
		add(selectedMovementCbBox, "1,3");
		add(movementsLst, "0,4,1,4");

		setBackground(WidgetColors.PROPERTIES_BACKGROUND);

	}

	private String getMovementStr(int i) {
		if (i < int2trans.length) {
			// sync move
			return (AlignmentConstants.MOVESYNC + " " + int2trans[i].getLabel());
		} else if (i < (2 * int2trans.length)) {
			// move model
			return (AlignmentConstants.MOVEMODELREAL + " " + int2trans[i - int2trans.length].getLabel());
		} else {
			// move log
			return (AlignmentConstants.MOVELOG + " " + int2ec[i - (2 * int2trans.length)].getId());
		}
	}

	private String getItemSetStr(int i) {
		String res = "";
		int[] itemSet = itemSets.getItemSets(i);
		for (int j = 0; j < itemSet.length; j++) {
			if (j != 0) {
				res += "-";
			}
			res += getMovementStr(itemSet[j]);
		}
		return res;
	}

	public void setItemSets(ItemSets itemSets) {
		this.itemSets = itemSets;

		// update combobox
		this.selectionCbBox.removeAllItems();
		if (itemSets.getNumItemSets() <= 0) {
			this.selectionCbBox.addItem("No pattern mined");
		} else {
			// add itemsets to combobox
			int numItemSets = itemSets.getNumItemSets();
			StringBuffer sb = new StringBuffer();
			for (int i = 0; i < numItemSets; i++) {
				sb.append("(sup: ");
				sb.append(itemSets.getSupport(i));
				sb.append(") ");
				sb.append(getItemSetStr(i));
				selectionCbBox.addItem((i + 1) + ". " + sb.toString());
				sb.delete(0, sb.length());
			}

			updateItemSetSelect(0);
		}
	}

	private void updateItemSetSelect(int itemSetID) {
		// pre
		itemsetsLst.invalidate();
		listModel.clear();

		if (itemSetID >= 0) {
			// insert all stats
			this.supportLbl.setText(String.valueOf(itemSets.getSupport(itemSetID)));
			int[] selectedItemset = itemSets.getItemSets(itemSetID);
			for (int encItem : selectedItemset) {
				listModel.addElement(getMovementStr(encItem));
			}

			// select movements in the graph
			parent.highlightGraph(parent.getViewSpecificAttributeMap(), selectedItemset);
		} else {
			// clear all stats
			this.supportLbl.setText("-");
			parent.highlightGraph(parent.getViewSpecificAttributeMap(), null);
		}
		// update all
		itemsetsLst.revalidate();
	}

}

class AlignmentRenderer extends DefaultListCellRenderer {
	private static final long serialVersionUID = 7381788002456208795L;

	public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected,
			boolean cellHasFocus) {
		super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
		String strVal = value.toString();
		if (strVal.startsWith(AlignmentConstants.MOVESYNC)) {
			setBackground(AlignmentConstants.MOVESYNCCOLOR);
			setForeground(Color.BLACK);
		} else if (strVal.startsWith(AlignmentConstants.MOVEMODELREAL)) {
			setBackground(AlignmentConstants.MOVEMODELREALCOLOR);
			setForeground(Color.BLACK);
		} else if (strVal.startsWith(AlignmentConstants.MOVELOG)) {
			setBackground(AlignmentConstants.MOVELOGCOLOR);
			setForeground(Color.BLACK);
		} else {
			setForeground(Color.WHITE);
		}
		setOpaque(true);
		return this;
	}
}

/**
 * List (JList) with SlickerBox L&F
 * 
 * @author aadrians
 * 
 *         modified from mwesterg
 * 
 *         TODO: change this to use ProMList as soon as there is a getter for
 *         jlist
 */
class ProMList2 extends RoundedPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private final JList jList;

	public ProMList2(final String title) {
		super(10, 5, 0);
		jList = new JList();
		setup(title);
	}

	public ProMList2(final String title, final ListModel providers) {
		super(10, 5, 0);
		jList = new JList(providers);
		setup(title);
	}

	public void addListSelectionListener(final ListSelectionListener l) {
		jList.addListSelectionListener(l);
	}

	@Override
	public void addMouseListener(final MouseListener l) {
		jList.addMouseListener(l);
	}

	/**
	 * @return
	 */
	public Object[] getSelectedValues() {
		return jList.getSelectedValues();
	}

	public void removeListSelectionListener(final ListSelectionListener l) {
		jList.removeListSelectionListener(l);
	}

	@Override
	public void removeMouseListener(final MouseListener l) {
		jList.removeMouseListener(l);
	}

	public void setSelectedIndex(final int index) {
		jList.setSelectedIndex(index);
	}

	/**
	 * @param selectedValues
	 */
	public void setSelection(final Iterable<Object> selectedValues) {
		jList.clearSelection();
		for (final Object value : selectedValues) {
			jList.setSelectedValue(value, true);
		}
	}

	/**
	 * @param selectedValues
	 */
	public void setSelection(final Object... selectedValues) {
		jList.clearSelection();
		for (final Object value : selectedValues) {
			jList.setSelectedValue(value, true);
		}
	}

	/**
	 * @param selectionMode
	 */
	public void setSelectionMode(final int selectionMode) {
		jList.setSelectionMode(selectionMode);
	}

	private void setup(final String title) {
		jList.setBackground(WidgetColors.COLOR_LIST_BG);
		jList.setForeground(WidgetColors.COLOR_LIST_FG);
		jList.setSelectionBackground(WidgetColors.COLOR_LIST_SELECTION_BG);
		jList.setSelectionForeground(WidgetColors.COLOR_LIST_SELECTION_FG);

		final JScrollPane scroller = new JScrollPane(jList);
		scroller.setOpaque(false);
		scroller.setBorder(BorderFactory.createEmptyBorder());
		scroller.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		scroller.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		final JScrollBar vBar = scroller.getVerticalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(160, 160, 160),
				WidgetColors.COLOR_NON_FOCUS, 4, 12));
		vBar.setOpaque(true);
		vBar.setBackground(WidgetColors.COLOR_ENCLOSURE_BG);

		final JLabel providersLabel = new JLabel(title);
		providersLabel.setOpaque(false);
		providersLabel.setForeground(WidgetColors.COLOR_LIST_SELECTION_FG);
		providersLabel.setFont(providersLabel.getFont().deriveFont(13f));
		providersLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		providersLabel.setHorizontalAlignment(SwingConstants.CENTER);
		providersLabel.setHorizontalTextPosition(SwingConstants.CENTER);

		setBackground(WidgetColors.COLOR_ENCLOSURE_BG);
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		add(providersLabel);
		add(Box.createVerticalStrut(8));
		add(scroller);
		setMinimumSize(new Dimension(200, 100));
		setMaximumSize(new Dimension(1000, 1000));
		setPreferredSize(new Dimension(1000, 200));
	}

	public JList getJList() {
		return jList;
	}
}
