/*
 * 3
 */
package org.processmining.plugins.pnalignanalysis.visualization.patternmining;

import info.clearthought.layout.TableLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;

import org.processmining.framework.util.ui.widgets.WidgetColors;
import org.processmining.plugins.pnalignanalysis.visualization.patternmining.algorithms.AbstractAssociationAlgorithm;
import org.processmining.plugins.pnalignanalysis.visualization.patternmining.algorithms.WekaFPGrowth;

import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * @author aadrians Jul 16, 2012
 * 
 */
public class MiningPanel extends JPanel {

	private static final long serialVersionUID = -589103827352287864L;

	private JComboBox algorithmCbBox;
	private JPanel parameter;
	private JProgressBar progressBar = new JProgressBar(JProgressBar.HORIZONTAL);


	public MiningPanel(final PNLogReplayPatternMinePanel parent) {
		SlickerFactory factory = SlickerFactory.instance();

		// combo box
		//Object[] algorithms = new Object[] { new WekaFPGrowth(), new WekaApriori() };
		Object[] algorithms = new Object[] { new WekaFPGrowth() };
		algorithmCbBox = factory.createComboBox(algorithms);

		parameter = new JPanel();
		final JLabel itemsetLbl = factory.createLabel("0 itemsets are found   ");

		// replace parameter according to selected algorithm 
		algorithmCbBox.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				parameter.removeAll();
				JPanel panel = ((AbstractAssociationAlgorithm) (algorithmCbBox.getSelectedItem())).getPanel();
				parameter.add(panel);
				parameter.repaint();
			}
		});
		parameter.setBackground(WidgetColors.PROPERTIES_BACKGROUND);

		// button
		final JButton mineBtn = factory.createButton("Mine Frequent Itemsets");
		mineBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				mineBtn.setEnabled(false);
				progressBar.setVisible(true);
				progressBar.setIndeterminate(true);
				new Thread(new ItemsetRunnable(mineBtn, parent, progressBar, itemsetLbl, ((AbstractAssociationAlgorithm) algorithmCbBox
						.getSelectedItem()))).start();
			}
		});

		// algorithm selection
		double size[][] = new double[][] { { 200, 300 }, { 30, TableLayout.FILL, 30, 10 } };
		setLayout(new TableLayout(size));
		add(factory.createLabel("Select algorithm"), "0,0");
		add(algorithmCbBox, "1,0");
		add(parameter, "0,1,1,1");
		add(mineBtn, "0,2");
		add(itemsetLbl, "1,2,r,c");
		add(progressBar, "0,3");
		progressBar.setIndeterminate(false);
		progressBar.setEnabled(false);
		progressBar.setVisible(false);

		setBackground(WidgetColors.PROPERTIES_BACKGROUND);

		// set selected index
		algorithmCbBox.setSelectedIndex(0);
		parameter.add(((AbstractAssociationAlgorithm) (algorithmCbBox.getSelectedItem())).getPanel());
		
	}
	
	public void startProgressBar(){
		progressBar.setVisible(true);
	}
	public void stopProgressBar(){
		progressBar.setVisible(false);
	}
}

class ItemsetRunnable implements Runnable {
	/**
	 * @param parent
	 * @param itemsetLbl
	 */
	final PNLogReplayPatternMinePanel parent;
	final JLabel itemsetLbl;
	final AbstractAssociationAlgorithm alg;
	final JProgressBar progressBar;
	final JButton mineBtn;

	public ItemsetRunnable(JButton mineBtn, PNLogReplayPatternMinePanel parent, final JProgressBar progressBar, final JLabel itemsetLbl,
			final AbstractAssociationAlgorithm alg) {
		this.mineBtn = mineBtn;
		this.parent = parent;
		this.progressBar = progressBar;
		this.itemsetLbl = itemsetLbl;
		this.alg = alg;
	}

	public void run() {
		ItemSets itemsets = alg.calculateItemSets(parent.getLogReplayResult(), parent.getInt2Trans(),
				parent.getInt2Ec());
		itemsetLbl.setText(itemsets.getNumItemSets() + " itemsets are found   ");

		parent.setItemsets(itemsets);
		progressBar.setIndeterminate(false);
		progressBar.setVisible(false);
		
		mineBtn.setEnabled(true);
	}

}
