/**
 * 
 */
package org.processmining.plugins.pnalignanalysis.visualization.patternmining;

import java.awt.BorderLayout;
import java.awt.Color;
import java.util.Arrays;
import java.util.Collection;
import java.util.Set;

import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XLog;
import org.jgraph.event.GraphSelectionEvent;
import org.jgraph.event.GraphSelectionListener;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.ViewSpecificAttributeMap;
import org.processmining.models.graphbased.directed.DirectedGraphNode;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.jgraph.ProMJGraph;
import org.processmining.models.jgraph.elements.ProMGraphCell;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.pnalignanalysis.visualization.projection.CaseFilterPanel;
import org.processmining.plugins.pnalignanalysis.visualization.projection.ExportPanel;
import org.processmining.plugins.pnalignanalysis.visualization.projection.PNLogReplayProjectedVisPanel;
import org.processmining.plugins.pnalignanalysis.visualization.projection.ProjectionVisPanel;
import org.processmining.plugins.pnalignanalysis.visualization.projection.TransConfDecorator;
import org.processmining.plugins.pnalignanalysis.visualization.projection.ViewPanel;
import org.processmining.plugins.pnalignanalysis.visualization.projection.util.GraphBuilder;

/**
 * @author aadrians Jul 15, 2012
 * 
 */
public class PNLogReplayPatternMinePanel extends ProjectionVisPanel {
	private static final long serialVersionUID = 3396089016889795055L;

	// GUI component
	private ViewPanel viewPanel;
	private MiningPanel minePanel;
	private float suggestedArcWidth = .75f;
	private float highlightedArcWidth = 1.0f;
	protected Color unhighlightedInviClr = new Color(200, 200, 200);

	// filter panel
	protected CaseFilterPanel caseFilterPanel;
	protected FreqItemsetsPanel patternPanel;

	// temporary
	private Transition[] int2trans;
	private XEventClass[] int2ec;

	public PNLogReplayPatternMinePanel(PluginContext context, PetrinetGraph origNet, Marking origMarking, XLog log,
			TransEvClassMapping map, PNRepResult logReplayResult) throws ConnectionCannotBeObtained {
		super(context, origNet, origMarking, log, map, logReplayResult);
		scalable = GraphBuilder.buildJGraph(origNet, oldLayoutConn);
		graph = (ProMJGraph) scalable;

		// encode transitions
		Collection<Transition> allTrans = origNet.getTransitions();
		int2trans = allTrans.toArray(new Transition[allTrans.size()]);

		// synchronous moves are encoded the same as the index of transitions
		// move on models are encoded as 2 x index of transitions

		// create mapping from event class to index
		// move on logs are encoded as (2xnum of transitions)
		XLogInfo logInfo = XLogInfoFactory.createLogInfo(log, map.getEventClassifier());
		Collection<XEventClass> evClasses = logInfo.getEventClasses().getClasses();
		int2ec = evClasses.toArray(new XEventClass[evClasses.size()]);

		colorGraphEdges(graph.getViewSpecificAttributes());

		/**
		 * TAB INFO
		 */
		/**
		 * Main visualization (has to be after creating provider)
		 */
		graph.addGraphSelectionListener(new GraphSelectionListener() {
			public void valueChanged(GraphSelectionEvent e) {
				// selection of a transition would change the stats
				if (e.getCell() instanceof ProMGraphCell) {
					DirectedGraphNode cell = ((ProMGraphCell) e.getCell()).getNode();
					if (cell instanceof Transition) {
						//statPanel.setTransition((Transition) cell);
					} else if (cell instanceof Place) {
						//statPanel.setPlace((Place) cell);
					}
				}
			}
		});

		for (Place p : marking) {
			String label = "" + marking.occurrences(p);
			graph.getViewSpecificAttributes().putViewSpecific(p, AttributeMap.LABEL, label);
			graph.getViewSpecificAttributes().putViewSpecific(p, AttributeMap.SHOWLABEL, !label.equals(""));
		}

		scroll = new JScrollPane(scalable.getComponent());
		decorator.decorate(scroll, Color.WHITE, Color.GRAY, Color.DARK_GRAY);
		setLayout(new BorderLayout());
		add(scroll);

		/**
		 * TAB DISPLAY SETTING
		 */
		// add additional tab for display settings
		JPanel displayMP = getInspector().addTab("Display");

		// add view panel (zoom in/out)
		viewPanel = createViewPanel(this, MAX_ZOOM);
		addInteractionViewports(viewPanel);

		// add elements
		getInspector().addGroup(displayMP, "View", viewPanel);

		// add association panel
		minePanel = createMiningPanel();
		addInfo("Mine Frequent Itemsets", minePanel);

		// add pattern panel
		patternPanel = createPatternPanel();
		addInfo("Frequent Itemsets", patternPanel);

		/**
		 * TAB EXPORT
		 */
		JPanel exportMP = getInspector().addTab("Export");
		getInspector().addGroup(exportMP, "Export", createExportPanel(graph));

		validate();
		repaint();
	}

	private ExportPanel createExportPanel(ProMJGraph scalable) {
		return new ExportPanel(graph);
	}

	private FreqItemsetsPanel createPatternPanel() {
		return new FreqItemsetsPanel(this);
	}

	private MiningPanel createMiningPanel() {
		return new MiningPanel(this);
	}

	public void filterAlignmentPreserveIndex(Set<Integer> preservedIndex) {
		// select the corresponding cases
		boolean[] filter = new boolean[this.log.size()];
		Arrays.fill(filter, false);
		for (int i : preservedIndex) {
			filter[i] = true;
		}
		caseFilterPanel.selectCases(filter);

		//TODO: reconstruct case related visualization

		validate();
		repaint();
	}

	public Transition[] getInt2Trans() {
		return int2trans;
	}

	public XEventClass[] getInt2Ec() {
		return int2ec;
	}

	/**
	 * This procedure is called after itemsets/association rules are mined
	 * 
	 * @param itemSets
	 */
	public void setItemsets(ItemSets itemSets) {
		// update patterns and show it
		patternPanel.setItemSets(itemSets);
	}

	/**
	 * Highlight elements (encoded) in the graph
	 * 
	 * @param selectedItemset
	 */
	public void highlightGraph(ViewSpecificAttributeMap map, int[] selectedElements) {
		graph.getModel().beginUpdate();
		/**
		 * Update main visualization (add decoration, size)
		 */
		resetAllGraphHighlights(map);

		if (selectedElements != null) {
			for (int i : selectedElements) {
				if (i < int2trans.length) {
					// synchronous moves
					// change the color according to decorator
					Transition t = int2trans[i];
					TransConfDecorator dec = decoratorMap.get(t);
					if (dec == null) {
						dec = new TransConfDecorator(0, 1, t.getLabel());
						decoratorMap.put(t, dec);
						map.putViewSpecific(t, AttributeMap.SHAPEDECORATOR, dec);
						dec.setLightColorLabel(true);
					} else {
						dec.setMoveSyncFreq(dec.getMoveSyncFreq() + 1);
						dec.setLightColorLabel(true);
					}
					map.putViewSpecific(t, AttributeMap.FILLCOLOR, PNLogReplayProjectedVisPanel.HIGH);
					map.putViewSpecific(t, AttributeMap.SHOWLABEL, false);

					highlightSurroundingElements(map, t);
				} else if (i < (2 * int2trans.length)) {
					// move on model
					Transition t = int2trans[i - int2trans.length];
					if (int2trans[i - int2trans.length].isInvisible()) {
						// invisible, change the color to black
						map.putViewSpecific(t, AttributeMap.FILLCOLOR, Color.BLACK);
						map.putViewSpecific(t, AttributeMap.LABELCOLOR, Color.WHITE);
					} else {
						// not invisible, change the color according to decorator
						TransConfDecorator dec = decoratorMap.get(t);
						if (dec == null) {
							dec = new TransConfDecorator(1, 0, t.getLabel());
							decoratorMap.put(t, dec);
							map.putViewSpecific(t, AttributeMap.SHAPEDECORATOR, dec);
							dec.setLightColorLabel(true);
						} else {
							dec.setMoveOnModelFreq(dec.getMoveOnModelFreq() + 1);
							dec.setLightColorLabel(true);
						}
						map.putViewSpecific(t, AttributeMap.FILLCOLOR, PNLogReplayProjectedVisPanel.HIGH);
						map.putViewSpecific(t, AttributeMap.SHOWLABEL, false);
					}

					highlightSurroundingElements(map, t);

				} else {
					// move logs
					// so far, do nothing
				}
			}
		}
		graph.getModel().endUpdate();
		graph.refresh();
		graph.revalidate();
		graph.repaint();
	}

	private void highlightSurroundingElements(ViewSpecificAttributeMap map, Transition t) {
		Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> edges = net.getInEdges(t);
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : edges) {
			map.putViewSpecific(edge, AttributeMap.EDGECOLOR, Color.BLACK);
			map.putViewSpecific(edge, AttributeMap.LINEWIDTH, highlightedArcWidth);
			map.putViewSpecific(edge, AttributeMap.STROKECOLOR, Color.BLACK);
			map.putViewSpecific(edge, AttributeMap.STROKE, highlightedArcWidth);
		}
		edges = net.getOutEdges(t);
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : edges) {
			map.putViewSpecific(edge, AttributeMap.EDGECOLOR, Color.BLACK);
			map.putViewSpecific(edge, AttributeMap.LINEWIDTH, highlightedArcWidth);
			map.putViewSpecific(edge, AttributeMap.STROKECOLOR, Color.BLACK);
			map.putViewSpecific(edge, AttributeMap.STROKE, highlightedArcWidth);
		}
	}

	private void colorGraphEdges(ViewSpecificAttributeMap map) {
		for (int i = 0; i < int2trans.length; i++) {
			Transition t = int2trans[i];
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> edges = net.getInEdges(t);

			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : edges) {
				map.putViewSpecific(edge, AttributeMap.EDGECOLOR, unhighlightedInviClr);
				map.putViewSpecific(edge, AttributeMap.LINEWIDTH, suggestedArcWidth);
			}
			edges = net.getOutEdges(t);
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : edges) {
				map.putViewSpecific(edge, AttributeMap.EDGECOLOR, unhighlightedInviClr);
				map.putViewSpecific(edge, AttributeMap.LINEWIDTH, suggestedArcWidth);
			}
		}
		for (PetrinetNode node : net.getNodes()) {
			map.putViewSpecific(node, AttributeMap.STROKECOLOR, unhighlightedInviClr);
			if (node instanceof Transition) {
				Transition t = (Transition) node;
				if (t.isInvisible()) {
					map.putViewSpecific(t, AttributeMap.FILLCOLOR, unhighlightedInviClr);
				}
				map.putViewSpecific(t, AttributeMap.SHOWLABEL, true);
			}
		}
	}

	private void resetAllGraphHighlights(ViewSpecificAttributeMap map) {
		for (Transition t : int2trans) {

			if (t.isInvisible()) {
				// invisible, color to light grey
				map.putViewSpecific(t, AttributeMap.SHOWLABEL, true);
				map.putViewSpecific(t, AttributeMap.LABELCOLOR, unhighlightedInviClr);
				map.putViewSpecific(t, AttributeMap.FILLCOLOR, unhighlightedInviClr);
				map.putViewSpecific(t, AttributeMap.SHAPEDECORATOR, null);
			} else {
				// not invisible, color to white

				map.putViewSpecific(t, AttributeMap.SHOWLABEL, true);
				map.putViewSpecific(t, AttributeMap.FILLCOLOR, PNLogReplayProjectedVisPanel.NOOCCURRENCE);
				map.putViewSpecific(t, AttributeMap.SHAPEDECORATOR, null);
			}
			map.putViewSpecific(t, AttributeMap.STROKECOLOR, unhighlightedInviClr);

			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> edges = net.getInEdges(t);
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : edges) {
				map.putViewSpecific(edge, AttributeMap.EDGECOLOR, unhighlightedInviClr);
				map.putViewSpecific(edge, AttributeMap.LINEWIDTH, suggestedArcWidth);
				map.putViewSpecific(edge, AttributeMap.STROKECOLOR, unhighlightedInviClr);
				map.putViewSpecific(edge, AttributeMap.STROKE, suggestedArcWidth);
			}
			edges = net.getOutEdges(t);
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : edges) {
				map.putViewSpecific(edge, AttributeMap.EDGECOLOR, unhighlightedInviClr);
				map.putViewSpecific(edge, AttributeMap.LINEWIDTH, suggestedArcWidth);
				map.putViewSpecific(edge, AttributeMap.STROKECOLOR, unhighlightedInviClr);
				map.putViewSpecific(edge, AttributeMap.STROKE, suggestedArcWidth);
			}
		}
	}

	public ViewSpecificAttributeMap getViewSpecificAttributeMap() {
		return graph.getViewSpecificAttributes();
	}

}
