/**
 * 
 */
package org.processmining.plugins.pnalignanalysis.visualization.patternmining.algorithms;

import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.JPanel;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.plugins.pnalignanalysis.visualization.patternmining.ItemSets;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;

/**
 * @author aadrians Jul 16, 2012
 * 
 */
public abstract class AbstractAssociationAlgorithm {
	public abstract JPanel getPanel();

	public abstract ItemSets calculateItemSets(PNRepResult logReplayResult, Transition[] int2trans,
			XEventClass[] int2ec);

	protected void createTempFileAndBuffer(final File temp, final BufferedWriter out, PNRepResult logReplayResult,
			Transition[] int2trans, XEventClass[] int2ec) throws IOException {
		out.write("@relation alignment");
		out.newLine();

		// map transition to integer
		Map<Transition, Integer> trans2int = new HashMap<Transition, Integer>();
		for (int i = 0; i < int2trans.length; i++) {
			trans2int.put(int2trans[i], i);
		}
		// map event class to integer
		Map<XEventClass, Integer> ec2Int = new HashMap<XEventClass, Integer>();
		for (int i = 0; i < int2ec.length; i++) {
			ec2Int.put(int2ec[i], i);
		}

		for (int i = 0; i < (2 * int2trans.length) + int2ec.length; i++) {
			out.write("@attribute ");
			out.write("" + i);
			out.write(" {0,1}");
			out.newLine();
		}

		out.write("@data");
		out.newLine();

		boolean[] flags = new boolean[(2 * int2trans.length) + int2ec.length];

		// print all attributes
		StringBuilder sb = new StringBuilder();

		for (SyncReplayResult repRes : logReplayResult) {
			// reset
			sb.delete(0, sb.length());
			Arrays.fill(flags, false);

			// obtain values
			List<Object> ni = repRes.getNodeInstance();
			List<StepTypes> st = repRes.getStepTypes();

			Iterator<StepTypes> stit = st.iterator();
			Iterator<Object> nist = ni.iterator();
			while (stit.hasNext()) {
				switch (stit.next()) {
					case L :
						flags[(int2trans.length * 2) + ec2Int.get(nist.next())] = true;
						break;
					case LMGOOD :
						flags[trans2int.get(nist.next())] = true;
						break;
					case MREAL :
					case MINVI :
						flags[int2trans.length + trans2int.get(nist.next())] = true;
						break;
					default :
						break;
				}
			}

			// print to temp file
			for (int i = 0; i < flags.length; i++) {
				if (i > 0) {
					sb.append(",");
				}
				if (flags[i]) {
					sb.append("1");
				} else {
					sb.append("0");
				}
			}

			String candidateStr = sb.toString();
			for (int i = 0; i < repRes.getTraceIndex().size(); i++) {
				out.write(candidateStr);
				out.newLine();
			}
		}
		out.close();
	}
}
