/**
 * 
 */
package org.processmining.plugins.pnalignanalysis.visualization.patternmining.algorithms;

import java.util.Comparator;
import java.util.Iterator;

import org.processmining.plugins.pnalignanalysis.visualization.patternmining.ItemSets;

import gnu.trove.list.array.TIntArrayList;
import weka.associations.BinaryItem;
import weka.associations.FPGrowth;

/**
 * @author aadrians Jul 17, 2012
 * 
 */
public class FPGrowthExt extends FPGrowth {
	private static final long serialVersionUID = -5844316464697822099L;

	public int getNumFrequentItemsets() {
		return this.m_largeItemSets.size();
	}

	public ItemSets getFrequentItemsets(int minItems, boolean keepClosedItemsets, int minSupport, int maxSupport) {
		// stores occurring movements
		TIntArrayList intArr = new TIntArrayList();

		// pointer to the length of patterns, followed by
		// support of the occurrence
		TIntArrayList ptrArr = new TIntArrayList();

		if (keepClosedItemsets) {
			// do the iteration backwards, because large itemsets are behind
			m_largeItemSets.sort(new Comparator<FPGrowth.FrequentBinaryItemSet>() {
				public int compare(FrequentBinaryItemSet o1, FrequentBinaryItemSet o2) {
					if (o1.getSupport() < o2.getSupport()) {
						return -1;
					} else if (o1.getSupport() > o2.getSupport()) {
						return 1;
					} else {
						if (o1.numberOfItems() < o2.numberOfItems()) {
							return -1;
						} else if (o1.numberOfItems() > o2.numberOfItems()) {
							return 1;
						}
						return 0;
					}
				}
			});

			int tsize = m_largeItemSets.size();
			FrequentBinaryItemSet[] explored = new FrequentBinaryItemSet[tsize];
			int idxExplored = 0;
			forLoop: for (int i = tsize - 1; i >= 0; i--) {
				FrequentBinaryItemSet itemset = m_largeItemSets.getItemSet(i);

				if ((itemset.getSupport() >= minSupport) && (itemset.getSupport() <= maxSupport)
						&& (itemset.numberOfItems() >= minItems)) {

					// iterate super in explored array
					for (int j = 0; j < idxExplored; j++) {
						if ((explored[j].getSupport() == itemset.getSupport())
								&& (explored[j].numberOfItems() > itemset.numberOfItems())) {
							// check if it is subsumed
							if (isSubsumed(itemset, explored[j])) {
								continue forLoop;
							}
						}
					}

					// add safely
					for (BinaryItem item : itemset.getItems()) {
						intArr.add(Integer.valueOf(item.getAttribute().name()));
					}
					intArr.add(-1);

					ptrArr.add(itemset.numberOfItems());
					ptrArr.add(itemset.getSupport());

					// add to explored array
					explored[idxExplored] = itemset;
					idxExplored++;
				}
			}

		} else {
			// just use iterator
			Iterator<FrequentBinaryItemSet> it = m_largeItemSets.iterator();
			while (it.hasNext()) {
				FrequentBinaryItemSet itemset = it.next();
				if ((itemset.getSupport() >= minSupport) && (itemset.getSupport() <= maxSupport)
						&& (itemset.numberOfItems() >= minItems)) {
					for (BinaryItem item : itemset.getItems()) {
						intArr.add(Integer.valueOf(item.getAttribute().name()));
					}
					intArr.add(-1);

					ptrArr.add(itemset.numberOfItems());
					ptrArr.add(itemset.getSupport());
				}
			}
		}

		// merge all stats into a single data
		int[] ptr = ptrArr.toArray();

		int[] data = new int[ptr.length + intArr.size()];

		// fill in the ptr
		int numItemsets = ptr.length / 2;
		int prevLength = 0;
		for (int i = 0; i < numItemsets; i++) {
			data[i] = ptr.length + prevLength;
			data[numItemsets + i] = ptr[(2 * i) + 1];
			prevLength += ptr[2 * i] + 1;
		}
		int[] arrRep = intArr.toArray();
		System.arraycopy(arrRep, 0, data, ptr.length, arrRep.length);

		return new ItemSets(data, ptr.length / 2);
	}

	/**
	 * return true if the subsumedItemset is a sub of itemset
	 * 
	 * @param itemset
	 * @param subsumedItemset
	 * @return
	 */
	private boolean isSubsumed(FrequentBinaryItemSet subsumedItemset, FrequentBinaryItemSet itemset) {
		return itemset.getItems().containsAll(subsumedItemset.getItems());
	}
}
