/**
 * 
 */
package org.processmining.plugins.pnalignanalysis.visualization.patternmining.algorithms;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.pnalignanalysis.visualization.patternmining.ItemSets;

import weka.associations.Apriori;
import weka.core.Instances;
import weka.core.converters.ConverterUtils.DataSource;
import weka.filters.Filter;
import weka.filters.unsupervised.attribute.Remove;

/**
 * @author aadrians
 * Jul 16, 2012
 *
 */
public class WekaApriori extends AbstractAssociationAlgorithm {
	public ItemSets calculateItemSets(PNRepResult logReplayResult, Transition[] int2trans, XEventClass[] int2ec){
		// get parameter from GUI
		return calculateItemSets(logReplayResult, int2trans, int2ec, WekaAprioriParam.getWekaAprioriParam());
	}
	
	
	public ItemSets calculateItemSets(PNRepResult logReplayResult, Transition[] int2trans, XEventClass[] int2ec, WekaAprioriParam param){
		// create temporary .arff file by encoding all movements
		File temp = null;
		BufferedWriter out = null;
		
		try {
			createTempFileAndBuffer(temp, out, logReplayResult, int2trans, int2ec); 
			
			// file is created, mine rules
			InputStream in = new FileInputStream(temp);
			DataSource source = new DataSource(in);
			Instances data = source.getDataSet();
			
			if (data.classIndex() == -1)
				data.setClassIndex(data.numAttributes() - 1);

			// generate data for clusterer (w/o class)
			Remove filter = new Remove();
			filter.setAttributeIndices("" + (data.classIndex() + 1));
			filter.setInputFormat(data);
			Instances dataClusterer = Filter.useFilter(data, filter);
			
			// apriori
			Apriori ap = new Apriori();
			String[] options = new String[] {"-N", "2", "-I", "yes"};
			ap.setOptions(options);
			ap.buildAssociations(dataClusterer);
			System.out.println(ap.toString());

			System.out.println("finish");
		} catch (IOException ioExc) {
			ioExc.printStackTrace();
		} catch (Exception exc) {
			exc.printStackTrace();
		} finally {
			if (out != null){
				try {
					out.close();
				} catch (Exception exc){
					exc.printStackTrace();
				}
			}
			if (temp != null){
				try {
					temp.delete();
				} catch (Exception exc){
					exc.printStackTrace();
				}
			}
		}
		return null;
	}
	
	public String toString(){
		return "Apriori Algorithm (WEKA)";
	}

	public WekaAprioriParamPanel getPanel() {
		return WekaAprioriParamPanel.getWekaAprioriParamPanel();
	}
}
