/**
 * 
 */
package org.processmining.plugins.pnalignanalysis.visualization.patternmining.algorithms;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.pnalignanalysis.visualization.patternmining.ItemSets;

import weka.core.Instances;
import weka.core.converters.ConverterUtils.DataSource;

/**
 * @author aadrians Jul 16, 2012
 * 
 */
public class WekaFPGrowth extends AbstractAssociationAlgorithm {
	public ItemSets calculateItemSets(PNRepResult logReplayResult, Transition[] int2trans,
			XEventClass[] int2ec) {
		// get parameter from GUI
		WekaFPGrowthParamPanel panel = getPanel();
		return calculateItemSets(logReplayResult, int2trans, int2ec, panel.getParameter());
	}

	public WekaFPGrowthParamPanel getPanel() {
		return WekaFPGrowthParamPanel.getWekaFPGrowthParamPanel();
	}

	public ItemSets calculateItemSets(PNRepResult logReplayResult, Transition[] int2trans, XEventClass[] int2ec,
			WekaFPGrowthParam param) {
		// create temporary .arff file by encoding all movements
		File temp = null;
		BufferedWriter out = null;

		try {
			temp = File.createTempFile("alignment", ".arff");
			out = new BufferedWriter(new FileWriter(temp));

			createTempFileAndBuffer(temp, out, logReplayResult, int2trans, int2ec);

			// file is created, mine rules
			InputStream in = new FileInputStream(temp);
			DataSource source = new DataSource(in);
			Instances data = source.getDataSet();

			// fp growth
			FPGrowthExt fp = new FPGrowthExt();
			fp.setOptions(param.generateWekaParamArray());
			fp.buildAssociations(data);
			
			int minSupport = (int) Math.round(param.getLowerBound() * data.numInstances());
			int maxSupport = (int) Math.round(param.getUpperBound() * data.numInstances());
			
			return fp.getFrequentItemsets(param.getMinItems(), param.isKeepClosedItemsets(), minSupport, maxSupport);
		} catch (IOException ioExc) {
			ioExc.printStackTrace();
		} catch (Exception exc) {
			exc.printStackTrace();
		} finally {
			if (out != null) {
				try {
					out.close();
				} catch (Exception exc) {
					exc.printStackTrace();
				}
			}
			if (temp != null) {
				try {
					temp.delete();
				} catch (Exception exc) {
					exc.printStackTrace();
				}
			}
		}
		return null;
	}

	public String toString() {
		return "FP-Growth Algorithm (WEKA)";
	}
}
