/**
 * 
 */
package org.processmining.plugins.pnalignanalysis.visualization.patternmining.algorithms;

/**
 * @author aadrians Jul 16, 2012
 * 
 */
public class WekaFPGrowthParam extends AbstractAssociationParam {
	public static WekaFPGrowthParam param = new WekaFPGrowthParam();

	private WekaFPGrowthParam() {
	};

	public static WekaFPGrowthParam getWekaFPGrowthParam() {
		return param;
	}

	/**
	 * The maximum number of items to include in large items sets (and rules).
	 */
	private int maxItems = -1;

	/**
	 * The minimum number of items to include in large items sets (and rules)
	 * added outside of WEKA
	 */
	private int minItems = 3;

	/**
	 * The required number of rules.
	 */
	//	private int numRules = 10;

	/**
	 * The metric by which to rank rules
	 */
	//	private int metricRankRule = 0;

	/**
	 * Minimum metric score of a rule
	 */
	//	private double minMetricScore = 0.9;

	/**
	 * Upper bound for minimum support.
	 */
	private double upperBound = 1.0;

	/**
	 * The lower bound for the minimum support.
	 */
	private double lowerBound = 0.5;

	/**
	 * The delta by which the minimum support is decreased in each iteration
	 */
	private double delta = 0.05;

	private boolean onlyKeepClosedItemsets = true;
	
	/**
	 * Find all rules that meet the lower bound on minimum support and the
	 * minimum metric constraint. Turning this mode on will disable the
	 * iterative support reduction procedure to find the specified number of
	 * rules.
	 */
	//	private boolean findAllRulesWithinBound = false;

	/**
	 * @return the maxItems
	 */
	public int getMaxItems() {
		return maxItems;
	}

	/**
	 * @param maxItems
	 *            the maxItems to set
	 */
	public void setMaxItems(int maxItems) {
		this.maxItems = maxItems;
	}

	public void setMinItems(int minItems) {
		this.minItems = minItems;
	}

	public int getMinItems() {
		return minItems;
	}

	/**
	 * @return the upperBound
	 */
	public double getUpperBound() {
		return upperBound;
	}

	/**
	 * @param upperBound
	 *            the upperBound to set
	 */
	public void setUpperBound(double upperBound) {
		this.upperBound = upperBound;
	}

	/**
	 * @return the lowerBound
	 */
	public double getLowerBound() {
		return lowerBound;
	}

	/**
	 * @param lowerBound
	 *            the lowerBound to set
	 */
	public void setLowerBound(double lowerBound) {
		this.lowerBound = lowerBound;
	}
	
	

	/**
	 * @return the delta
	 */
	public double getDelta() {
		return delta;
	}

	/**
	 * @param delta the delta to set
	 */
	public void setDelta(double delta) {
		this.delta = delta;
	}

	public String[] generateWekaParamArray() {
		String[] param;

		param = new String[15];
		param[0] = "-I"; // The maximum number of items to include in large items sets (and rules).
		param[1] = "" + maxItems;
		param[2] = "-U"; // Upper bound for minimum support
		param[3] = "" + upperBound;
		param[4] = "-M"; //  The lower bound for the minimum support.
		param[5] = "" + lowerBound;
		param[6] = "-D"; // The delta by which the minimum support is decreased in each iteration.
		param[7] = "" + delta;
		param[8] = "-N"; // The required number of rules.
		param[9] = "0";
		param[10] = "-T"; // The metric by which to rank rules. (confidence by default)
		param[11] = "0";
		param[12] = "-C"; // The minimum metric score of a rule.
		param[13] = "1.00";
		param[14] = "-S";
		
		//param[13] = ""  + lowerBound;
		return param;
	}

	public void setKeepClosedItemsets(boolean onlyKeepClosedItemsets) {
		this.onlyKeepClosedItemsets = onlyKeepClosedItemsets;
	}
	
	public boolean isKeepClosedItemsets(){
		return this.onlyKeepClosedItemsets;
	}

}
