/**
 * 
 */
package org.processmining.plugins.pnalignanalysis.visualization.patternmining.algorithms;

import info.clearthought.layout.TableLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;

import org.processmining.framework.util.ui.widgets.WidgetColors;

import com.fluxicon.slickerbox.components.NiceDoubleSlider;
import com.fluxicon.slickerbox.components.NiceIntegerSlider;
import com.fluxicon.slickerbox.components.NiceSlider.Orientation;
import com.fluxicon.slickerbox.factory.SlickerFactory;


/**
 * @author aadrians
 * Jul 16, 2012
 *
 */
public class WekaFPGrowthParamPanel extends AbstractAssociationParamPanel {
	private static final long serialVersionUID = 5357176059124070184L;

	private static WekaFPGrowthParamPanel panel = new WekaFPGrowthParamPanel();
	
	private NiceIntegerSlider minItemSlider;
	private NiceIntegerSlider maxItemSlider;
	private NiceDoubleSlider upperBoundSlider;
	private NiceDoubleSlider lowerBoundSlider;
//	private NiceDoubleSlider delta;
	private JCheckBox findClosedItemsets;
		
	private WekaFPGrowthParamPanel(){
		super();
		SlickerFactory factory = SlickerFactory.instance();
		minItemSlider = factory.createNiceIntegerSlider("", 0, 40, 3, Orientation.HORIZONTAL);
		maxItemSlider = factory.createNiceIntegerSlider("", 1, 100, 100, Orientation.HORIZONTAL);
		upperBoundSlider = factory.createNiceDoubleSlider("", 0.0, 1.0, 1.0, Orientation.HORIZONTAL);
		lowerBoundSlider = factory.createNiceDoubleSlider("", 0.0, 1.0, 0.9, Orientation.HORIZONTAL);
//		delta = factory.createNiceDoubleSlider("", 0.0, 1.0, 0.05, Orientation.HORIZONTAL);
		findClosedItemsets = factory.createCheckBox("Only keep closed itemsets", true);

		// set layout
		double size[][] = new double [][] {{200, 300 }, { 30, 30, 30, 30, 30, 30 }};
		setLayout(new TableLayout(size));
		add(factory.createLabel("Min #items in item sets"), "0,0");
		add(minItemSlider, "1,0");

		add(factory.createLabel("Max #items in item sets"), "0,1");
		add(maxItemSlider, "1,1");
		
//		add(factory.createLabel("Required #rules"), "0,1");
//		add(numRulesSlider, "1,1");
//		
//		add(factory.createLabel("Metric to rank rule"), "0,2");
//		add(metricRankRule, "1,2");
//		
//		add(factory.createLabel("Minimum metric score"), "0,3");
//		add(minMetricsScore, "1,3");
//		
		add(factory.createLabel("Upper bound of Min. support"), "0,2");
		add(upperBoundSlider, "1,2");
		
		add(factory.createLabel("Lower bound of Min. support"), "0,3");
		add(lowerBoundSlider, "1,3");
//		
//		add(factory.createLabel("Delta (for support iteration)"), "0,4");
//		add(delta, "1,4");
		
		add(findClosedItemsets, "0,4");
		
		JButton resetBtn = factory.createButton("Reset to default values");
		resetBtn.addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent e) {
				minItemSlider.setValue(3);
				maxItemSlider.setValue(100);
				upperBoundSlider.setValue(1.00);
				lowerBoundSlider.setValue(0.9);
				findClosedItemsets.setSelected(true);
			}
		});
		
		add(resetBtn, "1,4,r,c");
		
		setBackground(WidgetColors.PROPERTIES_BACKGROUND);

	}
	
	public static WekaFPGrowthParamPanel getWekaFPGrowthParamPanel(){
		return WekaFPGrowthParamPanel.panel;
	}
	
	public WekaFPGrowthParam getParameter() {
		WekaFPGrowthParam param = WekaFPGrowthParam.getWekaFPGrowthParam();
		param.setDelta(0.05);
//		param.setFindAllRulesWithinBound(isFindAllRulesWithinBound());
//		param.setMinMetricScore(getMinMetricScore());
//		param.setMetricRankRule(getMetricRankRule());
//		param.setNumRules(getNumRules());
		param.setLowerBound(getLowerBound());
		param.setUpperBound(getUpperBound());
		param.setMaxItems(getMaxItems());
		param.setMinItems(getMinItems());
		param.setKeepClosedItemsets(isOnlyKeepClosedItemsets());
		return param;
	}
	
	public int getMaxItems(){
		return maxItemSlider.getValue();
	}
	
	public int getMinItems(){
		return minItemSlider.getValue();
	}

	public double getLowerBound(){
		return lowerBoundSlider.getValue();
	}
	
	public double getUpperBound(){
		return upperBoundSlider.getValue();
	}
	
//	public double getDelta(){
//		return delta.getValue();
//	}
	
	public boolean isOnlyKeepClosedItemsets(){
		return findClosedItemsets.isSelected();
	}
	
	/**
	 * Find all rules that meet the lower bound on minimum support and the
	 * minimum metric constraint. Turning this mode on will disable the
	 * iterative support reduction procedure to find the specified number of
	 * rules.
	 */
//	public boolean isFindAllRulesWithinBound(){
//		return findAllRules.isSelected();
//	}

}
